@@ -8,20 +8,17 @@ example model ``bernoulli.stan``.
 
 Optional command line arguments:
    -v, --version <release> : version, defaults to latest release version
-   -d, --dir <path> : install directory, defaults to '$HOME/.cmdstan(py)
+   -d, --dir <path> : install directory, defaults to '$HOME/.cmdstan
    --overwrite: flag, when specified re-installs existing version
-   --verbose: flag, when specified prints output from CmdStan build process
    --progress: flag, when specified show progress bar for CmdStan download
-
+   --verbose: flag, when specified prints output from CmdStan build process
    -c, --compiler : flag, add C++ compiler to path (Windows only)
 """
 import argparse
-import contextlib
 import json
 import os
 import platform
 import re
-import subprocess
 import sys
 import tarfile
 import urllib.error
@@ -29,10 +26,24 @@ import urllib.request
 from collections import OrderedDict
 from pathlib import Path
 from time import sleep
+from typing import Any, Callable, Dict, Optional
 
-from cmdstanpy import _DOT_CMDSTAN, _DOT_CMDSTANPY
-from cmdstanpy.utils import validate_dir
+from tqdm.auto import tqdm
 
+from cmdstanpy import _DOT_CMDSTAN
+from cmdstanpy.utils import (
+    cmdstan_path,
+    do_command,
+    pushd,
+    validate_dir,
+    wrap_url_progress_hook,
+)
+
+from . import progress as progbar
+
+MAKE = os.getenv(
+    'MAKE', 'make' if platform.system() != 'Windows' else 'mingw32-make'
+)
 EXTENSION = '.exe' if platform.system() == 'Windows' else ''
 
 
@@ -44,24 +55,15 @@ class CmdStanInstallError(RuntimeError):
     pass
 
 
-@contextlib.contextmanager
-def pushd(new_dir: str):
-    """Acts like pushd/popd."""
-    previous_dir = os.getcwd()
-    os.chdir(new_dir)
-    yield
-    os.chdir(previous_dir)
-
-
-def usage():
+def usage() -> None:
     """Print usage."""
     msg = """
     Arguments:
         -v (--version) : CmdStan version
         -d (--dir) : install directory
         --overwrite : replace installed version
-        --verbose : show CmdStan build messages
         --progress : show progress bar for CmdStan download
+        --verbose : show outputs from installation processes
         """
 
     if platform.system() == "Windows":
@@ -72,9 +74,167 @@ def usage():
     print(msg)
 
 
+def clean_all(verbose: bool = False) -> None:
+    """
+    Run `make clean-all` in the current directory (must be a cmdstan library).
+
+    :param verbose: Boolean value; when ``True``, show output from make command.
+    """
+    cmd = [MAKE, 'clean-all']
+    try:
+        if verbose:
+            do_command(cmd)
+        else:
+            do_command(cmd, fd_out=None)
+
+    except RuntimeError as e:
+        # pylint: disable=raise-missing-from
+        raise CmdStanInstallError(f'Command "make clean-all" failed\n{str(e)}')
+
+
+def build(verbose: bool = False, progress: bool = True, cores: int = 1) -> None:
+    """
+    Run command ``make build`` in the current directory, which must be
+    the home directory of a CmdStan version (or GitHub repo).
+    By default, displays a progress bar which tracks make command outputs.
+    If argument ``verbose=True``, instead of a progress bar, streams
+    make command outputs to sys.stdout.  When both ``verbose`` and ``progress``
+    are ``False``, runs silently.
+
+    :param verbose: Boolean value; when ``True``, show output from make command.
+        Default is ``False``.
+    :param progress: Boolean value; when ``True`` display progress progress bar.
+        Default is ``True``.
+    :param cores: Integer, number of cores to use in the ``make`` command.
+        Default is 1 core.
+    """
+    cmd = [MAKE, 'build', f'-j{cores}']
+    try:
+        if verbose:
+            do_command(cmd)
+        elif progress and progbar.allow_show_progress():
+            progress_hook: Any = _wrap_build_progress_hook()
+            do_command(cmd, fd_out=None, pbar=progress_hook)
+        else:
+            do_command(cmd, fd_out=None)
+
+    except RuntimeError as e:
+        # pylint: disable=raise-missing-from
+        raise CmdStanInstallError(f'Command "make build" failed\n{str(e)}')
+    if not os.path.exists(os.path.join('bin', 'stansummary' + EXTENSION)):
+        raise CmdStanInstallError(
+            f'bin/stansummary{EXTENSION} not found'
+            ', please rebuild or report a bug!'
+        )
+    if not os.path.exists(os.path.join('bin', 'diagnose' + EXTENSION)):
+        raise CmdStanInstallError(
+            f'bin/stansummary{EXTENSION} not found'
+            ', please rebuild or report a bug!'
+        )
+
+    if platform.system() == 'Windows':
+        # Add tbb to the $PATH on Windows
+        libtbb = os.path.join(
+            os.getcwd(), 'stan', 'lib', 'stan_math', 'lib', 'tbb'
+        )
+        os.environ['PATH'] = ';'.join(
+            list(
+                OrderedDict.fromkeys(
+                    [libtbb] + os.environ.get('PATH', '').split(';')
+                )
+            )
+        )
+
+
+@progbar.wrap_callback
+def _wrap_build_progress_hook() -> Optional[Callable[[str], None]]:
+    """Sets up tqdm callback for CmdStan sampler console msgs."""
+    pad = ' ' * 20
+    msgs_expected = 150  # hack: 2.27 make build send ~140 msgs to console
+    pbar: tqdm = tqdm(
+        total=msgs_expected,
+        bar_format="{desc} ({elapsed}) | {bar} | {postfix[0][value]}",
+        postfix=[dict(value=f'Building CmdStan {pad}')],
+        colour='blue',
+        desc='',
+        position=0,
+    )
+
+    def build_progress_hook(line: str) -> None:
+        if line.startswith('--- CmdStan'):
+            pbar.set_description('Done')
+            pbar.postfix[0]["value"] = line
+            pbar.update(msgs_expected - pbar.n)
+            pbar.close()
+        else:
+            if line.startswith('--'):
+                pbar.postfix[0]["value"] = line
+            else:
+                pbar.postfix[0]["value"] = f'{line[:8]} ... {line[-20:]}'
+                pbar.set_description('Compiling')
+                pbar.update(1)
+
+    return build_progress_hook
+
+
+def compile_example(verbose: bool = False) -> None:
+    """
+    Compile the example model.
+    The current directory must be a cmdstan installation, i.e.,
+    contains the makefile, Stanc compiler, and all libraries.
+
+    :param verbose: Boolean value; when ``True``, show output from make command.
+    """
+    cmd = [
+        MAKE,
+        Path(
+            os.path.join('examples', 'bernoulli', 'bernoulli' + EXTENSION)
+        ).as_posix(),
+    ]
+    try:
+        if verbose:
+            do_command(cmd)
+        else:
+            do_command(cmd, fd_out=None)
+    except RuntimeError as e:
+        # pylint: disable=raise-missing-from
+        raise CmdStanInstallError(f'Command "make clean-all" failed\n{e}')
+
+
+def rebuild_cmdstan(
+    verbose: bool = False, progress: bool = True, cores: int = 1
+) -> None:
+    """
+    Rebuilds the existing CmdStan installation.
+    This assumes CmdStan has already been installed,
+    though it need not be installed via CmdStanPy for
+    this function to work.
+
+    :param verbose: Boolean value; when ``True``, show output from make command.
+        Default is ``False``.
+    :param progress: Boolean value; when ``True`` display progress progress bar.
+        Default is ``True``.
+    :param cores: Integer, number of cores to use in the ``make`` command.
+        Default is 1 core.
+    """
+    try:
+        with pushd(cmdstan_path()):
+            clean_all(verbose)
+            build(verbose, progress, cores)
+            compile_example(verbose)
+    except ValueError as e:
+        raise CmdStanInstallError(
+            "Failed to rebuild CmdStan. Are you sure it is installed?"
+        ) from e
+
+
 def install_version(
-    cmdstan_version: str, overwrite: bool = False, verbose: bool = False
-):
+    cmdstan_version: str,
+    overwrite: bool = False,
+    verbose: bool = False,
+    progress: bool = True,
+    cores: int = 1,
+) -> None:
     """
     Build specified CmdStan version by spawning subprocesses to
     run the Make utility on the downloaded CmdStan release src files.
@@ -82,96 +242,27 @@ def install_version(
 
     :param cmdstan_version: CmdStan release, corresponds to release dirname.
     :param overwrite: when ``True``, run ``make clean-all`` before building.
-    :param verbose: when ``True``, print build msgs to stdout.
+    :param verbose: Boolean value; when ``True``, show output from make command.
     """
     with pushd(cmdstan_version):
-        make = os.getenv(
-            'MAKE', 'make' if platform.system() != 'Windows' else 'mingw32-make'
+        print(
+            'Building version {}, may take several minutes, '
+            'depending on your system.'.format(cmdstan_version)
         )
-        print('Building version {}'.format(cmdstan_version))
         if overwrite:
             print(
                 'Overwrite requested, remove existing build of version '
                 '{}'.format(cmdstan_version)
             )
-            cmd = [make, 'clean-all']
-            proc = subprocess.Popen(
-                cmd,
-                cwd=None,
-                stdin=subprocess.DEVNULL,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-                env=os.environ,
-            )
-            while proc.poll() is None:
-                output = proc.stdout.readline().decode('utf-8').strip()
-                if verbose and output:
-                    print(output, flush=True)
-            _, stderr = proc.communicate()
-            if proc.returncode:
-                msgs = ['Command "make clean-all" failed']
-                if stderr:
-                    msgs.append(stderr.decode('utf-8').strip())
-                raise CmdStanInstallError('\n'.join(msgs))
+            clean_all(verbose)
             print('Rebuilding version {}'.format(cmdstan_version))
-        cmd = [make, 'build']
-        proc = subprocess.Popen(
-            cmd,
-            cwd=None,
-            stdin=subprocess.DEVNULL,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            env=os.environ,
-        )
-        while proc.poll() is None:
-            output = proc.stdout.readline().decode('utf-8').strip()
-            if verbose and output:
-                print(output, flush=True)
-        _, stderr = proc.communicate()
-        if proc.returncode:
-            msgs = ['Command "make build" failed']
-            if stderr:
-                msgs.append(stderr.decode('utf-8').strip())
-            raise CmdStanInstallError('\n'.join(msgs))
+        build(verbose, progress=progress, cores=cores)
         print('Test model compilation')
-        cmd = [
-            make,
-            Path(
-                os.path.join('examples', 'bernoulli', 'bernoulli' + EXTENSION)
-            ).as_posix(),
-        ]
-        if platform.system() == "Windows":
-            # Add tbb to the $PATH on Windows
-            libtbb = os.path.join(
-                os.getcwd(), 'stan', 'lib', 'stan_math', 'lib', 'tbb'
-            )
-            os.environ['PATH'] = ';'.join(
-                list(
-                    OrderedDict.fromkeys(
-                        [libtbb] + os.environ.get('PATH', '').split(';')
-                    )
-                )
-            )
-        proc = subprocess.Popen(
-            cmd,
-            cwd=None,
-            stdin=subprocess.DEVNULL,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
-            env=os.environ,
-        )
-        while proc.poll() is None:
-            proc.stdout.readline().decode('utf-8')
-        _, stderr = proc.communicate()
-        if proc.returncode:
-            msgs = ['Failed to compile example model bernoulli.stan']
-            if stderr:
-                msgs.append(stderr.decode('utf-8').strip())
-            raise CmdStanInstallError('\n'.join(msgs))
+        compile_example(verbose)
     print('Installed {}'.format(cmdstan_version))
 
 
-def is_version_available(version: str):
+def is_version_available(version: str) -> bool:
     is_available = True
     url = (
         'https://github.com/stan-dev/cmdstan/releases/download/'
@@ -181,8 +272,8 @@ def is_version_available(version: str):
         try:
             urllib.request.urlopen(url)
         except urllib.error.HTTPError as err:
-            print('Release {} is unavailable from URL {}'.format(version, url))
-            print('HTTPError: {}'.format(err.code))
+            print(f'Release {version} is unavailable from URL {url}')
+            print(f'HTTPError: {err.code}')
             is_available = False
             break
         except urllib.error.URLError as e:
@@ -200,7 +291,7 @@ def is_version_available(version: str):
     return is_available
 
 
-def get_headers():
+def get_headers() -> Dict[str, str]:
     """Create headers dictionary."""
     headers = {}
     GITHUB_PAT = os.environ.get("GITHUB_PAT")  # pylint:disable=invalid-name
@@ -209,7 +300,7 @@ def get_headers():
     return headers
 
 
-def latest_version():
+def latest_version() -> str:
     """Report latest CmdStan release version."""
     url = 'https://api.github.com/repos/stan-dev/cmdstan/releases/latest'
     request = urllib.request.Request(url, headers=get_headers())
@@ -232,36 +323,10 @@ def latest_version():
     match = re.search(r'v?(.+)', tag)
     if match is not None:
         tag = match.group(1)
-    return tag
+    return tag  # type: ignore
 
 
-def wrap_progress_hook():
-    try:
-        from tqdm import tqdm
-
-        pbar = tqdm(
-            unit='B',
-            unit_scale=True,
-            unit_divisor=1024,
-        )
-
-        def download_progress_hook(count, block_size, total_size):
-            if pbar.total is None:
-                pbar.total = total_size
-                pbar.reset()
-            downloaded_size = count * block_size
-            pbar.update(downloaded_size - pbar.n)
-            if pbar.n >= total_size:
-                pbar.close()
-
-    except (ImportError, ModuleNotFoundError):
-        print("tqdm is not installed, progressbar not shown")
-        download_progress_hook = None
-
-    return download_progress_hook
-
-
-def retrieve_version(version: str, progress=True):
+def retrieve_version(version: str, progress: bool = True) -> None:
     """Download specified CmdStan version."""
     if version is None or version == '':
         raise ValueError('Argument "version" unspecified.')
@@ -272,8 +337,10 @@ def retrieve_version(version: str, progress=True):
     )
     for i in range(6):  # always retry to allow for transient URLErrors
         try:
-            if progress:
-                progress_hook = wrap_progress_hook()
+            if progress and progbar.allow_show_progress():
+                progress_hook: Optional[
+                    Callable[[int, int, int], None]
+                ] = wrap_url_progress_hook()
             else:
                 progress_hook = None
             file_tmp, _ = urllib.request.urlretrieve(
@@ -304,29 +371,48 @@ def retrieve_version(version: str, progress=True):
             ) from e
     print('Download successful, file: {}'.format(file_tmp))
     try:
+        print('Extracting distribution')
         tar = tarfile.open(file_tmp)
+        first = tar.next()
+        if first is not None:
+            top_dir = first.name
+        cmdstan_dir = f'cmdstan-{version}'
+        if top_dir != cmdstan_dir:
+            raise CmdStanInstallError(
+                'tarfile should contain top-level dir {},'
+                'but found dir {} instead.'.format(cmdstan_dir, top_dir)
+            )
         target = os.getcwd()
         if platform.system() == 'Windows':
             # fixes long-path limitation on Windows
             target = r'\\?\{}'.format(target)
-        tar.extractall(target)
+
+        if progress and progbar.allow_show_progress():
+            for member in tqdm(
+                iterable=tar.getmembers(),
+                total=len(tar.getmembers()),
+                colour='blue',
+                leave=False,
+            ):
+                tar.extract(member=member)
+        else:
+            tar.extractall()
     except Exception as e:  # pylint: disable=broad-except
         raise CmdStanInstallError(
-            'Failed to unpack file {}'.format(file_tmp)
+            f'Failed to unpack file {file_tmp}, error:\n\t{str(e)}'
         ) from e
     finally:
         tar.close()
-    print('Unpacked download as cmdstan-{}'.format(version))
+    print(f'Unpacked download as {cmdstan_dir}')
 
 
-def main():
-    """Main."""
+def parse_cmdline_args() -> Dict[str, Any]:
     parser = argparse.ArgumentParser()
     parser.add_argument(
         '--version', '-v', help="version, defaults to latest release version"
     )
     parser.add_argument(
-        '--dir', '-d', help="install directory, defaults to '$HOME/.cmdstan(py)"
+        '--dir', '-d', help="install directory, defaults to '$HOME/.cmdstan"
     )
     parser.add_argument(
         '--overwrite',
@@ -343,6 +429,12 @@ def main():
         action='store_true',
         help="flag, when specified show progress bar for CmdStan download",
     )
+    parser.add_argument(
+        "--cores",
+        default=1,
+        type=int,
+        help="number of cores to use while building",
+    )
     if platform.system() == 'Windows':
         # use compiler installed with install_cxx_toolchain
         # Install a new compiler if compiler not found
@@ -354,43 +446,40 @@ def main():
             action='store_true',
             help="flag, add C++ compiler to path (Windows only)",
         )
-    args = parser.parse_args(sys.argv[1:])
+    return vars(parser.parse_args(sys.argv[1:]))
+
+
+def main(args: Dict[str, Any]) -> None:
+    """Main."""
 
     version = latest_version()
-    if vars(args)['version']:
-        version = vars(args)['version']
+    if args['version']:
+        version = args['version']
 
     if is_version_available(version):
         print('Installing CmdStan version: {}'.format(version))
     else:
         raise ValueError(
-            'Invalid version requested: {}, cannot install.'.format(version)
+            f'Version {version} cannot be downloaded. '
+            'Connection to GitHub failed. '
+            'Check firewall settings or ensure this version exists.'
         )
 
     cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
-    if not os.path.exists(cmdstan_dir):
-        cmdstanpy_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTANPY))
-        if os.path.exists(cmdstanpy_dir):
-            cmdstan_dir = cmdstanpy_dir
 
     install_dir = cmdstan_dir
-    if vars(args)['dir']:
-        install_dir = vars(args)['dir']
+    if args['dir']:
+        install_dir = args['dir']
 
     validate_dir(install_dir)
     print('Install directory: {}'.format(install_dir))
 
-    if vars(args)['progress']:
-        progress = vars(args)['progress']
-        try:
-            # pylint: disable=unused-import
-            from tqdm import tqdm  # noqa: F401
-        except (ImportError, ModuleNotFoundError):
-            progress = False
+    if args['progress']:
+        progress = args['progress']
     else:
         progress = False
 
-    if platform.system() == 'Windows' and vars(args)['compiler']:
+    if platform.system() == 'Windows' and args['compiler']:
         from .install_cxx_toolchain import is_installed as _is_installed_cxx
         from .install_cxx_toolchain import main as _main_cxx
         from .utils import cxx_toolchain_path
@@ -416,19 +505,15 @@ def main():
         if not compiler_found:
             print('Installing RTools40')
             # copy argv and clear sys.argv
-            original_argv = sys.argv[:]
-            sys.argv = [
-                item for item in sys.argv if item not in ("--compiler", "-c")
-            ]
-            _main_cxx()
-            sys.argv = original_argv
+            cxx_args = {k: v for k, v in args.items() if k != 'compiler'}
+            _main_cxx(cxx_args)
             cxx_version = '40'
         # Add toolchain to $PATH
-        cxx_toolchain_path(cxx_version)
+        cxx_toolchain_path(cxx_version, args['dir'])
 
-    cmdstan_version = 'cmdstan-{}'.format(version)
+    cmdstan_version = f'cmdstan-{version}'
     with pushd(install_dir):
-        if vars(args)['overwrite'] or not (
+        if args['overwrite'] or not (
             os.path.exists(cmdstan_version)
             and os.path.exists(
                 os.path.join(
@@ -443,8 +528,10 @@ def main():
                 retrieve_version(version, progress)
                 install_version(
                     cmdstan_version=cmdstan_version,
-                    overwrite=vars(args)['overwrite'],
-                    verbose=vars(args)['verbose'],
+                    overwrite=args['overwrite'],
+                    verbose=args['verbose'],
+                    progress=progress,
+                    cores=args['cores'],
                 )
             except RuntimeError as e:
                 print(e)
@@ -453,5 +540,9 @@ def main():
             print('CmdStan version {} already installed'.format(version))
 
 
+def __main__() -> None:
+    main(parse_cmdline_args())
+
+
 if __name__ == '__main__':
-    main()
+    __main__()
