@@ -13,7 +13,6 @@ Optional command line arguments:
    --progress : flag, when specified show progress bar for RTools download
 """
 import argparse
-import contextlib
 import os
 import platform
 import shutil
@@ -22,24 +21,16 @@ import sys
 import urllib.request
 from collections import OrderedDict
 from time import sleep
+from typing import Any, Dict, List
 
-from cmdstanpy import _DOT_CMDSTAN, _DOT_CMDSTANPY
-from cmdstanpy.utils import validate_dir
+from cmdstanpy import _DOT_CMDSTAN
+from cmdstanpy.utils import pushd, validate_dir, wrap_url_progress_hook
 
 EXTENSION = '.exe' if platform.system() == 'Windows' else ''
 IS_64BITS = sys.maxsize > 2 ** 32
 
 
-@contextlib.contextmanager
-def pushd(new_dir):
-    """Acts like pushd/popd."""
-    previous_dir = os.getcwd()
-    os.chdir(new_dir)
-    yield
-    os.chdir(previous_dir)
-
-
-def usage():
+def usage() -> None:
     """Print usage."""
     print(
         """Arguments:
@@ -53,7 +44,7 @@ def usage():
     )
 
 
-def get_config(dir, silent):
+def get_config(dir: str, silent: bool) -> List[str]:
     """Assemble config info."""
     config = []
     if platform.system() == 'Windows':
@@ -74,8 +65,12 @@ def get_config(dir, silent):
 
 
 def install_version(
-    installation_dir, installation_file, version, silent, verbose=False
-):
+    installation_dir: str,
+    installation_file: str,
+    version: str,
+    silent: bool,
+    verbose: bool = False,
+) -> None:
     """Install specified toolchain version."""
     with pushd('.'):
         print(
@@ -95,9 +90,10 @@ def install_version(
             env=os.environ,
         )
         while proc.poll() is None:
-            output = proc.stdout.readline().decode('utf-8').strip()
-            if output and verbose:
-                print(output, flush=True)
+            if proc.stdout:
+                output = proc.stdout.readline().decode('utf-8').strip()
+                if output and verbose:
+                    print(output, flush=True)
         _, stderr = proc.communicate()
         if proc.returncode:
             print('Installation failed: returncode={}'.format(proc.returncode))
@@ -112,7 +108,7 @@ def install_version(
     print('Installed {}'.format(os.path.splitext(installation_file)[0]))
 
 
-def install_mingw32_make(toolchain_loc, verbose=False):
+def install_mingw32_make(toolchain_loc: str, verbose: bool = False) -> None:
     """Install mingw32-make for Windows RTools 4.0."""
     os.environ['PATH'] = ';'.join(
         list(
@@ -146,9 +142,10 @@ def install_mingw32_make(toolchain_loc, verbose=False):
             env=os.environ,
         )
         while proc.poll() is None:
-            output = proc.stdout.readline().decode('utf-8').strip()
-            if output and verbose:
-                print(output, flush=True)
+            if proc.stdout:
+                output = proc.stdout.readline().decode('utf-8').strip()
+                if output and verbose:
+                    print(output, flush=True)
         _, stderr = proc.communicate()
         if proc.returncode:
             print(
@@ -162,7 +159,7 @@ def install_mingw32_make(toolchain_loc, verbose=False):
     print('Installed mingw32-make.exe')
 
 
-def is_installed(toolchain_loc, version):
+def is_installed(toolchain_loc: str, version: str) -> bool:
     """Returns True is toolchain is installed."""
     if platform.system() == 'Windows':
         if version in ['35', '3.5']:
@@ -190,46 +187,20 @@ def is_installed(toolchain_loc, version):
     return False
 
 
-def latest_version():
+def latest_version() -> str:
     """Windows version hardcoded to 4.0."""
     if platform.system() == 'Windows':
         return '4.0'
     return ''
 
 
-def wrap_progress_hook():
-    try:
-        from tqdm import tqdm
-
-        pbar = tqdm(
-            unit='B',
-            unit_scale=True,
-            unit_divisor=1024,
-        )
-
-        def download_progress_hook(count, block_size, total_size):
-            if pbar.total is None:
-                pbar.total = total_size
-                pbar.reset()
-            downloaded_size = count * block_size
-            pbar.update(downloaded_size - pbar.n)
-            if pbar.n >= total_size:
-                pbar.close()
-
-    except (ImportError, ModuleNotFoundError):
-        print("tqdm was not downloaded, progressbar not shown")
-        download_progress_hook = None
-
-    return download_progress_hook
-
-
-def retrieve_toolchain(filename, url, progress=True):
+def retrieve_toolchain(filename: str, url: str, progress: bool = True) -> None:
     """Download toolchain from URL."""
     print('Downloading C++ toolchain: {}'.format(filename))
     for i in range(6):
         try:
             if progress:
-                progress_hook = wrap_progress_hook()
+                progress_hook = wrap_url_progress_hook()
             else:
                 progress_hook = None
             _ = urllib.request.urlretrieve(
@@ -247,7 +218,7 @@ def retrieve_toolchain(filename, url, progress=True):
     print('Download successful, file: {}'.format(filename))
 
 
-def normalize_version(version):
+def normalize_version(version: str) -> str:
     """Return maj.min part of version string."""
     if platform.system() == 'Windows':
         if version in ['4', '40']:
@@ -257,14 +228,14 @@ def normalize_version(version):
     return version
 
 
-def get_toolchain_name():
+def get_toolchain_name() -> str:
     """Return toolchain name."""
     if platform.system() == 'Windows':
         return 'RTools'
     return ''
 
 
-def get_url(version):
+def get_url(version: str) -> str:
     """Return URL for toolchain."""
     if platform.system() == 'Windows':
         if version == '4.0':
@@ -278,24 +249,87 @@ def get_url(version):
     return url
 
 
-def get_toolchain_version(name, version):
+def get_toolchain_version(name: str, version: str) -> str:
     """Toolchain version."""
-    toolchain_folder = None
+    toolchain_folder = ''
     if platform.system() == 'Windows':
         toolchain_folder = '{}{}'.format(name, version.replace('.', ''))
 
     return toolchain_folder
 
 
-def main():
+def main(args: Dict[str, Any]) -> None:
     """Main."""
     if platform.system() not in {'Windows'}:
-        msg = (
-            'Download for the C++ toolchain'
-            ' on the current platform has not been implemented: %s'
+        raise NotImplementedError(
+            'Download for the C++ toolchain '
+            'on the current platform has not '
+            f'been implemented: {platform.system()}'
         )
-        raise NotImplementedError(msg % platform.system())
+    toolchain = get_toolchain_name()
+    version = args['version']
+    if version is None:
+        version = latest_version()
+    version = normalize_version(version)
+    print("C++ toolchain '{}' version: {}".format(toolchain, version))
 
+    url = get_url(version)
+
+    if 'verbose' in args:
+        verbose = args['verbose']
+
+    install_dir = args['dir']
+    if install_dir is None:
+        install_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
+    validate_dir(install_dir)
+    print('Install directory: {}'.format(install_dir))
+
+    if 'progress' in args:
+        progress = args['progress']
+    else:
+        progress = False
+
+    if platform.system() == 'Windows':
+        silent = 'silent' in args
+        # force silent == False for 4.0 version
+        if 'silent' not in args and version in ('4.0', '4', '40'):
+            silent = False
+    else:
+        silent = False
+
+    toolchain_folder = get_toolchain_version(toolchain, version)
+    with pushd(install_dir):
+        if is_installed(toolchain_folder, version):
+            print('C++ toolchain {} already installed'.format(toolchain_folder))
+        else:
+            if os.path.exists(toolchain_folder):
+                shutil.rmtree(toolchain_folder, ignore_errors=False)
+            retrieve_toolchain(
+                toolchain_folder + EXTENSION, url, progress=progress
+            )
+            install_version(
+                toolchain_folder,
+                toolchain_folder + EXTENSION,
+                version,
+                silent,
+                verbose,
+            )
+        if (
+            'no-make' not in args
+            and (platform.system() == 'Windows')
+            and (version in ('4.0', '4', '40'))
+        ):
+            if os.path.exists(
+                os.path.join(
+                    toolchain_folder, 'mingw64', 'bin', 'mingw32-make.exe'
+                )
+            ):
+                print('mingw32-make.exe already installed')
+            else:
+                install_mingw32_make(toolchain_folder, verbose)
+
+
+def parse_cmdline_args() -> Dict[str, Any]:
     parser = argparse.ArgumentParser()
     parser.add_argument('--version', '-v', help="version, defaults to latest")
     parser.add_argument(
@@ -323,82 +357,12 @@ def main():
         action='store_true',
         help="flag, when specified show progress bar for CmdStan download",
     )
-    args = parser.parse_args(sys.argv[1:])
+    return vars(parser.parse_args(sys.argv[1:]))
 
-    toolchain = get_toolchain_name()
-    version = vars(args)['version']
-    if version is None:
-        version = latest_version()
-    version = normalize_version(version)
-    print("C++ toolchain '{}' version: {}".format(toolchain, version))
 
-    url = get_url(version)
-
-    if 'verbose' in vars(args):
-        verbose = vars(args)['verbose']
-
-    install_dir = vars(args)['dir']
-    if install_dir is None:
-        cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
-        if not os.path.exists(cmdstan_dir):
-            cmdstanpy_dir = os.path.expanduser(
-                os.path.join('~', _DOT_CMDSTANPY)
-            )
-            if os.path.exists(cmdstanpy_dir):
-                cmdstan_dir = cmdstanpy_dir
-        install_dir = cmdstan_dir
-    validate_dir(install_dir)
-    print('Install directory: {}'.format(install_dir))
-
-    if 'progress' in vars(args):
-        progress = vars(args)['progress']
-        try:
-            # pylint: disable=unused-import
-            from tqdm import tqdm  # noqa: F401
-        except (ImportError, ModuleNotFoundError):
-            progress = False
-    else:
-        progress = False
-
-    if platform.system() == 'Windows':
-        silent = 'silent' in vars(args)
-        # force silent == False for 4.0 version
-        if 'silent' not in vars(args) and version in ('4.0', '4', '40'):
-            silent = False
-    else:
-        silent = False
-
-    toolchain_folder = get_toolchain_version(toolchain, version)
-    with pushd(install_dir):
-        if is_installed(toolchain_folder, version):
-            print('C++ toolchain {} already installed'.format(toolchain_folder))
-        else:
-            if os.path.exists(toolchain_folder):
-                shutil.rmtree(toolchain_folder, ignore_errors=False)
-            retrieve_toolchain(
-                toolchain_folder + EXTENSION, url, progress=progress
-            )
-            install_version(
-                toolchain_folder,
-                toolchain_folder + EXTENSION,
-                version,
-                silent,
-                verbose,
-            )
-        if (
-            'no-make' not in vars(args)
-            and (platform.system() == 'Windows')
-            and (version in ('4.0', '4', '40'))
-        ):
-            if os.path.exists(
-                os.path.join(
-                    toolchain_folder, 'mingw64', 'bin', 'mingw32-make.exe'
-                )
-            ):
-                print('mingw32-make.exe already installed')
-            else:
-                install_mingw32_make(toolchain_folder, verbose)
+def __main__() -> None:
+    main(parse_cmdline_args())
 
 
 if __name__ == '__main__':
-    main()
+    __main__()
