@@ -6,7 +6,7 @@ import unittest
 from cmdstanpy import _TMPDIR
 from cmdstanpy.cmdstan_args import CmdStanArgs, SamplerArgs
 from cmdstanpy.stanfit import RunSet
-from cmdstanpy.utils import EXTENSION, cmdstan_version_at
+from cmdstanpy.utils import EXTENSION
 
 HERE = os.path.dirname(os.path.abspath(__file__))
 DATAFILES_PATH = os.path.join(HERE, 'data')
@@ -25,13 +25,13 @@ class RunSetTest(unittest.TestCase):
             data=jdata,
             method_args=sampler_args,
         )
-        runset = RunSet(args=cmdstan_args)
-
+        runset = RunSet(args=cmdstan_args, chains=4)
         self.assertIn('RunSet: chains=4', runset.__repr__())
         self.assertIn('method=sample', runset.__repr__())
         self.assertIn('retcodes=[-1, -1, -1, -1]', runset.__repr__())
-        self.assertIn('csv_files:', runset.__repr__())
-        self.assertNotIn('diagnostics_files:', runset.__repr__())
+        self.assertIn('csv_file', runset.__repr__())
+        self.assertIn('console_msgs', runset.__repr__())
+        self.assertNotIn('diagnostics_file', runset.__repr__())
 
     def test_check_retcodes(self):
         exe = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
@@ -45,7 +45,7 @@ class RunSetTest(unittest.TestCase):
             data=jdata,
             method_args=sampler_args,
         )
-        runset = RunSet(args=cmdstan_args)
+        runset = RunSet(args=cmdstan_args, chains=4)
 
         retcodes = runset._retcodes
         self.assertEqual(4, len(retcodes))
@@ -73,22 +73,13 @@ class RunSetTest(unittest.TestCase):
             method_args=sampler_args,
         )
         runset = RunSet(args=cmdstan_args, chains=3, chain_ids=chain_ids)
-        if not cmdstan_version_at(2, 27):
-            for i in range(3):
-                runset._set_retcode(i, 70)
-                stdout_file = 'chain-' + str(i + 1) + '-missing-data-stdout.txt'
-                path = os.path.join(DATAFILES_PATH, stdout_file)
-                runset._stdout_files[i] = path
-            errs = runset.get_err_msgs()
-            self.assertIn('Exception: variable does not exist', errs)
-        else:
-            for i in range(3):
-                runset._set_retcode(i, 1)
-                stderr_file = 'chain-' + str(i + 1) + '-missing-data-stderr.txt'
-                path = os.path.join(DATAFILES_PATH, stderr_file)
-                runset._stderr_files[i] = path
-            errs = runset.get_err_msgs()
-            self.assertIn('Exception: variable does not exist', errs)
+        for i in range(3):
+            runset._set_retcode(i, 70)
+            stdout_file = 'chain-' + str(i + 1) + '-missing-data-stdout.txt'
+            path = os.path.join(DATAFILES_PATH, stdout_file)
+            runset._stdout_files[i] = path
+        errs = runset.get_err_msgs()
+        self.assertIn('Exception: variable does not exist', errs)
 
     def test_output_filenames(self):
         exe = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
@@ -102,10 +93,10 @@ class RunSetTest(unittest.TestCase):
             data=jdata,
             method_args=sampler_args,
         )
-        runset = RunSet(args=cmdstan_args)
+        runset = RunSet(args=cmdstan_args, chains=4)
         self.assertIn('bernoulli-', runset._csv_files[0])
-        self.assertIn('-1-', runset._csv_files[0])
-        self.assertIn('-4-', runset._csv_files[3])
+        self.assertIn('_1.csv', runset._csv_files[0])
+        self.assertIn('_4.csv', runset._csv_files[3])
 
     def test_commands(self):
         exe = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
@@ -119,11 +110,11 @@ class RunSetTest(unittest.TestCase):
             data=jdata,
             method_args=sampler_args,
         )
-        runset = RunSet(args=cmdstan_args)
-        self.assertIn('id=1', runset._cmds[0])
-        self.assertIn('id=4', runset._cmds[3])
+        runset = RunSet(args=cmdstan_args, chains=4)
+        self.assertIn('id=1', runset.cmd(0))
+        self.assertIn('id=4', runset.cmd(3))
 
-    def test_save_diagnostics(self):
+    def test_save_latent_dynamics(self):
         exe = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
         sampler_args = SamplerArgs()
@@ -134,9 +125,9 @@ class RunSetTest(unittest.TestCase):
             chain_ids=chain_ids,
             data=jdata,
             method_args=sampler_args,
-            save_diagnostics=True,
+            save_latent_dynamics=True,
         )
-        runset = RunSet(args=cmdstan_args)
+        runset = RunSet(args=cmdstan_args, chains=4)
         self.assertIn(_TMPDIR, runset.diagnostic_files[0])
 
         cmdstan_args = CmdStanArgs(
@@ -145,10 +136,10 @@ class RunSetTest(unittest.TestCase):
             chain_ids=chain_ids,
             data=jdata,
             method_args=sampler_args,
-            save_diagnostics=True,
+            save_latent_dynamics=True,
             output_dir=os.path.abspath('.'),
         )
-        runset = RunSet(args=cmdstan_args)
+        runset = RunSet(args=cmdstan_args, chains=4)
         self.assertIn(os.path.abspath('.'), runset.diagnostic_files[0])
 
     def test_chain_ids(self):
@@ -164,27 +155,10 @@ class RunSetTest(unittest.TestCase):
             method_args=sampler_args,
         )
         runset = RunSet(args=cmdstan_args, chains=4, chain_ids=chain_ids)
-        self.assertIn('id=11', runset._cmds[0])
-        self.assertIn('-11-', runset._csv_files[0])
-        self.assertIn('id=14', runset._cmds[3])
-        self.assertIn('-14-', runset._csv_files[3])
-
-    def test_ctor_checks(self):
-        exe = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
-        jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
-        sampler_args = SamplerArgs()
-        chain_ids = [11, 12, 13, 14]
-        cmdstan_args = CmdStanArgs(
-            model_name='bernoulli',
-            model_exe=exe,
-            chain_ids=chain_ids,
-            data=jdata,
-            method_args=sampler_args,
-        )
-        with self.assertRaises(ValueError):
-            RunSet(args=cmdstan_args, chains=0)
-        with self.assertRaises(ValueError):
-            RunSet(args=cmdstan_args, chains=4, chain_ids=[1, 2, 3])
+        self.assertIn('id=11', runset.cmd(0))
+        self.assertIn('_11.csv', runset._csv_files[0])
+        self.assertIn('id=14', runset.cmd(3))
+        self.assertIn('_14.csv', runset._csv_files[3])
 
 
 if __name__ == '__main__':
