@@ -1,6 +1,10 @@
 """utils test"""
 
+import collections.abc
+import contextlib
+import io
 import json
+import logging
 import os
 import platform
 import random
@@ -9,79 +13,75 @@ import stat
 import string
 import tempfile
 import unittest
+from pathlib import Path
+from test import CustomTestCase
 
 import numpy as np
+import pandas as pd
+import pytest
+from testfixtures import LogCapture
 
-from cmdstanpy import _DOT_CMDSTAN, _DOT_CMDSTANPY, _TMPDIR
+from cmdstanpy import _DOT_CMDSTAN, _TMPDIR
 from cmdstanpy.model import CmdStanModel
+from cmdstanpy.progress import _disable_progress, allow_show_progress
 from cmdstanpy.utils import (
+    EXTENSION,
+    BaseType,
     MaybeDictToFilePath,
-    TemporaryCopiedFile,
+    SanitizedOrTmpFilePath,
     check_sampler_csv,
     cmdstan_path,
-    cmdstan_version_at,
+    cmdstan_version,
+    cmdstan_version_before,
     do_command,
+    flatten_chains,
     get_latest_cmdstan,
-    jsondump,
+    parse_method_vars,
     parse_rdump_value,
-    parse_sampler_vars,
     parse_stan_vars,
-    rdump,
+    pushd,
     read_metric,
     rload,
     set_cmdstan_path,
     validate_cmdstan_path,
     validate_dir,
     windows_short_path,
+    write_stan_json,
 )
 
 HERE = os.path.dirname(os.path.abspath(__file__))
 DATAFILES_PATH = os.path.join(HERE, 'data')
+BERN_STAN = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+BERN_DATA = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+BERN_EXE = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
 
 
-class CmdStanPathTest(unittest.TestCase):
+class CmdStanPathTest(CustomTestCase):
     def test_default_path(self):
-        cur_value = None
         if 'CMDSTAN' in os.environ:
-            cur_value = os.environ['CMDSTAN']
-        try:
-            if 'CMDSTAN' in os.environ:
-                self.assertEqual(cmdstan_path(), os.environ['CMDSTAN'])
-                path = os.environ['CMDSTAN']
-                del os.environ['CMDSTAN']
+            self.assertPathsEqual(cmdstan_path(), os.environ['CMDSTAN'])
+            path = os.environ['CMDSTAN']
+            with self.modified_environ('CMDSTAN'):
                 self.assertFalse('CMDSTAN' in os.environ)
                 set_cmdstan_path(path)
-                self.assertEqual(cmdstan_path(), path)
+                self.assertPathsEqual(cmdstan_path(), path)
                 self.assertTrue('CMDSTAN' in os.environ)
-            else:
-                cmdstan_dir = os.path.expanduser(
-                    os.path.join('~', _DOT_CMDSTAN)
-                )
-                if not os.path.exists(cmdstan_dir):
-                    cmdstan_dir = os.path.expanduser(
-                        os.path.join('~', _DOT_CMDSTANPY)
-                    )
-                install_version = os.path.join(
-                    cmdstan_dir, get_latest_cmdstan(cmdstan_dir)
-                )
-                self.assertTrue(
-                    os.path.samefile(cmdstan_path(), install_version)
-                )
-                self.assertTrue('CMDSTAN' in os.environ)
-        finally:
-            if cur_value is not None:
-                os.environ['CMDSTAN'] = cur_value
-            else:
-                if 'CMDSTAN' in os.environ:
-                    del os.environ['CMDSTAN']
+        else:
+            cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
+            install_version = os.path.join(
+                cmdstan_dir, get_latest_cmdstan(cmdstan_dir)
+            )
+            self.assertTrue(os.path.samefile(cmdstan_path(), install_version))
+            self.assertTrue('CMDSTAN' in os.environ)
 
     def test_non_spaces_location(self):
         with tempfile.TemporaryDirectory(
             prefix="cmdstan_tests", dir=_TMPDIR
         ) as tmpdir:
             good_path = os.path.join(tmpdir, 'good_dir')
-            with TemporaryCopiedFile(good_path) as (pth, is_changed):
-                self.assertEqual(pth, good_path)
+            os.mkdir(good_path)
+            with SanitizedOrTmpFilePath(good_path) as (pth, is_changed):
+                self.assertPathsEqual(pth, good_path)
                 self.assertFalse(is_changed)
 
             # prepare files for test
@@ -93,7 +93,7 @@ class CmdStanPathTest(unittest.TestCase):
 
             stan_copied = None
             try:
-                with TemporaryCopiedFile(stan_bad) as (pth, is_changed):
+                with SanitizedOrTmpFilePath(stan_bad) as (pth, is_changed):
                     stan_copied = pth
                     self.assertTrue(os.path.exists(stan_copied))
                     self.assertTrue(' ' not in stan_copied)
@@ -106,35 +106,35 @@ class CmdStanPathTest(unittest.TestCase):
                 self.assertFalse(os.path.exists(stan_copied))
 
             # cleanup after test
+            shutil.rmtree(good_path, ignore_errors=True)
             shutil.rmtree(bad_path, ignore_errors=True)
 
     def test_set_path(self):
         if 'CMDSTAN' in os.environ:
-            self.assertEqual(cmdstan_path(), os.environ['CMDSTAN'])
+            self.assertPathsEqual(cmdstan_path(), os.environ['CMDSTAN'])
         else:
             cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
-            if not os.path.exists(cmdstan_dir):
-                cmdstan_dir = os.path.expanduser(
-                    os.path.join('~', _DOT_CMDSTANPY)
-                )
             install_version = os.path.join(
                 cmdstan_dir, get_latest_cmdstan(cmdstan_dir)
             )
             set_cmdstan_path(install_version)
-            self.assertEqual(install_version, cmdstan_path())
-            self.assertEqual(install_version, os.environ['CMDSTAN'])
+            self.assertPathsEqual(install_version, cmdstan_path())
+            self.assertPathsEqual(install_version, os.environ['CMDSTAN'])
 
     def test_validate_path(self):
-        cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
-        if not os.path.exists(cmdstan_dir):
-            cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTANPY))
-        install_version = os.path.join(
-            cmdstan_dir, get_latest_cmdstan(cmdstan_dir)
-        )
+        if 'CMDSTAN' in os.environ:
+            install_version = os.environ.get('CMDSTAN')
+        else:
+            cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
+
+            install_version = os.path.join(
+                cmdstan_dir, get_latest_cmdstan(cmdstan_dir)
+            )
+
         set_cmdstan_path(install_version)
         validate_cmdstan_path(install_version)
         path_foo = os.path.abspath(os.path.join('releases', 'foo'))
-        with self.assertRaisesRegex(ValueError, 'no such CmdStan directory'):
+        with self.assertRaisesRegex(ValueError, 'No CmdStan directory'):
             validate_cmdstan_path(path_foo)
         folder_name = ''.join(
             random.choice(string.ascii_letters) for _ in range(10)
@@ -145,7 +145,7 @@ class CmdStanPathTest(unittest.TestCase):
             )
         os.makedirs(folder_name)
         path_test = os.path.abspath(folder_name)
-        with self.assertRaisesRegex(ValueError, 'no CmdStan binaries'):
+        with self.assertRaisesRegex(ValueError, 'missing binaries'):
             validate_cmdstan_path(path_test)
         shutil.rmtree(folder_name)
 
@@ -183,9 +183,44 @@ class CmdStanPathTest(unittest.TestCase):
             os.makedirs(os.path.join(tdir, 'cmdstan-2.22.0'))
             self.assertEqual(get_latest_cmdstan(tdir), 'cmdstan-2.22.0')
 
-    def test_cmdstan_version_at(self):
+    def test_cmdstan_version_before(self):
         cmdstan_path()  # sets os.environ['CMDSTAN']
-        self.assertFalse(cmdstan_version_at(99, 99))
+        self.assertTrue(cmdstan_version_before(99, 99))
+        self.assertFalse(cmdstan_version_before(1, 1))
+
+    def test_cmdstan_version(self):
+        with tempfile.TemporaryDirectory(
+            prefix="cmdstan_tests", dir=_TMPDIR
+        ) as tmpdir:
+            tdir = os.path.join(tmpdir, 'tmpdir_xxx')
+            os.makedirs(tdir)
+            fake_path = os.path.join(tdir, 'cmdstan-2.22.0')
+            os.makedirs(os.path.join(fake_path))
+            fake_bin = os.path.join(fake_path, 'bin')
+            os.makedirs(fake_bin)
+            Path(os.path.join(fake_bin, 'stanc' + EXTENSION)).touch()
+            with self.modified_environ(CMDSTAN=fake_path):
+                self.assertTrue(fake_path == cmdstan_path())
+                expect = (
+                    'CmdStan installation {} missing makefile, '
+                    'cannot get version.'.format(fake_path)
+                )
+                with LogCapture() as log:
+                    logging.getLogger()
+                    cmdstan_version()
+                log.check_present(('cmdstanpy', 'INFO', expect))
+                fake_makefile = os.path.join(fake_path, 'makefile')
+                with open(fake_makefile, 'w') as fd:
+                    fd.write('...  CMDSTAN_VERSION := dont_need_no_mmp\n\n')
+                expect = (
+                    'Cannot parse version, expected "<major>.<minor>.<patch>", '
+                    'found: "dont_need_no_mmp".'
+                )
+                with LogCapture() as log:
+                    logging.getLogger()
+                    cmdstan_version()
+                log.check_present(('cmdstanpy', 'INFO', expect))
+        cmdstan_path()
 
 
 class DataFilesTest(unittest.TestCase):
@@ -206,44 +241,71 @@ class DataFilesTest(unittest.TestCase):
             with MaybeDictToFilePath(123, dict_good) as (fg1, fg2):
                 pass
 
-    def test_jsondump(self):
+    def test_write_stan_json(self):
         def cmp(d1, d2):
             self.assertEqual(d1.keys(), d2.keys())
             for k in d1:
                 data_1 = d1[k]
                 data_2 = d2[k]
-                if isinstance(data_2, np.ndarray):
-                    data_2 = data_2.tolist()
+                if isinstance(data_2, collections.abc.Collection):
+                    data_2 = np.asarray(data_2).tolist()
+
                 self.assertEqual(data_1, data_2)
 
         dict_list = {'a': [1.0, 2.0, 3.0]}
         file_list = os.path.join(_TMPDIR, 'list.json')
-        jsondump(file_list, dict_list)
+        write_stan_json(file_list, dict_list)
         with open(file_list) as fd:
             cmp(json.load(fd), dict_list)
 
-        dict_vec = {'a': np.repeat(3, 4)}
+        arr = np.repeat(3, 4)
+        dict_vec = {'a': arr}
         file_vec = os.path.join(_TMPDIR, 'vec.json')
-        jsondump(file_vec, dict_vec)
+        write_stan_json(file_vec, dict_vec)
         with open(file_vec) as fd:
             cmp(json.load(fd), dict_vec)
 
+        dict_bool = {'a': False}
+        file_bool = os.path.join(_TMPDIR, 'bool.json')
+        write_stan_json(file_bool, dict_bool)
+        with open(file_bool) as fd:
+            cmp(json.load(fd), {'a': 0})
+
+        dict_none = {'a': None}
+        file_none = os.path.join(_TMPDIR, 'none.json')
+        write_stan_json(file_none, dict_none)
+        with open(file_none) as fd:
+            cmp(json.load(fd), dict_none)
+
+        series = pd.Series(arr)
+        dict_vec_pd = {'a': series}
+        file_vec_pd = os.path.join(_TMPDIR, 'vec_pd.json')
+        write_stan_json(file_vec_pd, dict_vec_pd)
+        with open(file_vec_pd) as fd:
+            cmp(json.load(fd), dict_vec_pd)
+
+        df_vec = pd.DataFrame(dict_list)
+        file_pd = os.path.join(_TMPDIR, 'pd.json')
+        write_stan_json(file_pd, df_vec)
+        with open(file_pd) as fd:
+            cmp(json.load(fd), dict_list)
+
         dict_zero_vec = {'a': []}
         file_zero_vec = os.path.join(_TMPDIR, 'empty_vec.json')
-        jsondump(file_zero_vec, dict_zero_vec)
+        write_stan_json(file_zero_vec, dict_zero_vec)
         with open(file_zero_vec) as fd:
             cmp(json.load(fd), dict_zero_vec)
 
         dict_zero_matrix = {'a': [[], [], []]}
         file_zero_matrix = os.path.join(_TMPDIR, 'empty_matrix.json')
-        jsondump(file_zero_matrix, dict_zero_matrix)
+        write_stan_json(file_zero_matrix, dict_zero_matrix)
         with open(file_zero_matrix) as fd:
             cmp(json.load(fd), dict_zero_matrix)
 
         arr = np.zeros(shape=(5, 0))
         dict_zero_matrix = {'a': arr}
         file_zero_matrix = os.path.join(_TMPDIR, 'empty_matrix.json')
-        jsondump(file_zero_matrix, dict_zero_matrix)
+        write_stan_json(file_zero_matrix, dict_zero_matrix)
         with open(file_zero_matrix) as fd:
             cmp(json.load(fd), dict_zero_matrix)
 
@@ -253,7 +315,7 @@ class DataFilesTest(unittest.TestCase):
 
         dict_3d_matrix = {'a': arr}
         file_3d_matrix = os.path.join(_TMPDIR, '3d_matrix.json')
-        jsondump(file_3d_matrix, dict_3d_matrix)
+        write_stan_json(file_3d_matrix, dict_3d_matrix)
         with open(file_3d_matrix) as fd:
             cmp(json.load(fd), dict_3d_matrix)
 
@@ -261,12 +323,51 @@ class DataFilesTest(unittest.TestCase):
         self.assertTrue(type(scalr).__module__ == 'numpy')
         dict_scalr = {'a': scalr}
         file_scalr = os.path.join(_TMPDIR, 'scalr.json')
-        jsondump(file_scalr, dict_scalr)
+        write_stan_json(file_scalr, dict_scalr)
         with open(file_scalr) as fd:
             cmp(json.load(fd), dict_scalr)
 
+        # custom Stan serialization
+        dict_inf_nan = {
+            'a': np.array(
+                [
+                    [-np.inf, np.inf, np.NaN],
+                    [-float('inf'), float('inf'), float('NaN')],
+                    [
+                        np.float32(-np.inf),
+                        np.float32(np.inf),
+                        np.float32(np.NaN),
+                    ],
+                    [1e200 * -1e200, 1e220 * 1e200, -np.nan],
+                ]
+            )
+        }
+        dict_inf_nan_exp = {'a': [["-inf", "+inf", "NaN"]] * 4}
+        file_fin = os.path.join(_TMPDIR, 'inf.json')
+        write_stan_json(file_fin, dict_inf_nan)
+        with open(file_fin) as fd:
+            cmp(json.load(fd), dict_inf_nan_exp)
 
-class ReadStanCsvTest(unittest.TestCase):
+        dict_complex = {'a': np.array([np.complex64(3), 3 + 4j])}
+        dict_complex_exp = {'a': [[3, 0], [3, 4]]}
+        file_complex = os.path.join(_TMPDIR, 'complex.json')
+        write_stan_json(file_complex, dict_complex)
+        with open(file_complex) as fd:
+            cmp(json.load(fd), dict_complex_exp)
+
+    def test_write_stan_json_bad(self):
+        file_bad = os.path.join(_TMPDIR, 'bad.json')
+
+        dict_badtype = {'a': 'a string'}
+        with self.assertRaises(TypeError):
+            write_stan_json(file_bad, dict_badtype)
+
+        dict_badtype_nested = {'a': ['a string']}
+        with self.assertRaises(ValueError):
+            write_stan_json(file_bad, dict_badtype_nested)
+
+
+class ReadStanCsvTest(CustomTestCase):
     def test_check_sampler_csv_1(self):
         csv_good = os.path.join(DATAFILES_PATH, 'bernoulli_output_1.csv')
         dict = check_sampler_csv(
@@ -281,13 +382,13 @@ class ReadStanCsvTest(unittest.TestCase):
         self.assertEqual(10, dict['draws_sampling'])
         self.assertEqual(8, len(dict['column_names']))
 
-        with self.assertRaisesRegex(
+        with self.assertRaisesRegexNested(
             ValueError, 'config error, expected thin = 2'
         ):
             check_sampler_csv(
                 path=csv_good, iter_warmup=100, iter_sampling=20, thin=2
             )
-        with self.assertRaisesRegex(
+        with self.assertRaisesRegexNested(
             ValueError, 'config error, expected save_warmup'
         ):
             check_sampler_csv(
@@ -296,7 +397,7 @@ class ReadStanCsvTest(unittest.TestCase):
                 iter_sampling=10,
                 save_warmup=True,
             )
-        with self.assertRaisesRegex(ValueError, 'expected 1000 draws'):
+        with self.assertRaisesRegexNested(ValueError, 'expected 1000 draws'):
             check_sampler_csv(path=csv_good, iter_warmup=100)
 
     def test_check_sampler_csv_2(self):
@@ -306,34 +407,34 @@ class ReadStanCsvTest(unittest.TestCase):
 
     def test_check_sampler_csv_3(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_cols.csv')
-        with self.assertRaisesRegex(Exception, '8 items'):
+        with self.assertRaisesRegexNested(Exception, '8 items'):
             check_sampler_csv(csv_bad)
 
     def test_check_sampler_csv_4(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_rows.csv')
-        with self.assertRaisesRegex(Exception, 'found 9'):
+        with self.assertRaisesRegexNested(Exception, 'found 9'):
             check_sampler_csv(csv_bad)
 
     def test_check_sampler_csv_metric_1(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_metric_1.csv')
-        with self.assertRaisesRegex(Exception, 'expecting metric'):
+        with self.assertRaisesRegexNested(Exception, 'expecting metric'):
             check_sampler_csv(csv_bad)
 
     def test_check_sampler_csv_metric_2(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_metric_2.csv')
-        with self.assertRaisesRegex(Exception, 'invalid step size'):
+        with self.assertRaisesRegexNested(Exception, 'invalid step size'):
             check_sampler_csv(csv_bad)
 
     def test_check_sampler_csv_metric_3(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_metric_3.csv')
-        with self.assertRaisesRegex(
+        with self.assertRaisesRegexNested(
             Exception, 'invalid or missing mass matrix specification'
         ):
             check_sampler_csv(csv_bad)
 
     def test_check_sampler_csv_metric_4(self):
         csv_bad = os.path.join(DATAFILES_PATH, 'output_bad_metric_4.csv')
-        with self.assertRaisesRegex(
+        with self.assertRaisesRegexNested(
             Exception, 'invalid or missing mass matrix specification'
         ):
             check_sampler_csv(csv_bad)
@@ -369,7 +470,7 @@ class ReadStanCsvTest(unittest.TestCase):
         self.assertEqual(dict['max_depth'], 11)
         self.assertEqual(dict['delta'], 0.98)
 
-        with self.assertRaisesRegex(ValueError, 'config error'):
+        with self.assertRaisesRegexNested(ValueError, 'config error'):
             check_sampler_csv(
                 path=csv_file,
                 is_fixed_param=False,
@@ -377,7 +478,9 @@ class ReadStanCsvTest(unittest.TestCase):
                 iter_warmup=490,
                 thin=9,
             )
-        with self.assertRaisesRegex(ValueError, 'expected 490 draws, found 70'):
+        with self.assertRaisesRegexNested(
+            ValueError, 'expected 490 draws, found 70'
+        ):
             check_sampler_csv(
                 path=csv_file,
                 is_fixed_param=False,
@@ -534,21 +637,6 @@ class RloadTest(unittest.TestCase):
         with self.assertRaises(ValueError):
             rload(dfile)
 
-    def test_roundtrip_metric(self):
-        dfile = os.path.join(DATAFILES_PATH, 'metric_diag.data.R')
-        data_dict_1 = rload(dfile)
-        self.assertEqual(data_dict_1['inv_metric'].shape, (3,))
-
-        dfile_tmp = os.path.join(DATAFILES_PATH, 'tmp.data.R')
-        rdump(dfile_tmp, data_dict_1)
-        data_dict_2 = rload(dfile_tmp)
-
-        self.assertTrue('inv_metric' in data_dict_2)
-        for i, x in enumerate(data_dict_2['inv_metric']):
-            self.assertEqual(x, data_dict_2['inv_metric'][i])
-
-        os.remove(dfile_tmp)
-
     def test_parse_rdump_value(self):
         struct1 = (
             'structure(c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16),.Dim=c(2,8))'
@@ -578,19 +666,20 @@ class RloadTest(unittest.TestCase):
 class ParseVarsTest(unittest.TestCase):
     def test_parse_empty(self):
         x = []
-        sampler_vars = parse_sampler_vars(x)
+        sampler_vars = parse_method_vars(x)
         self.assertEqual(len(sampler_vars), 0)
-        stan_vars_dims, stan_vars_cols = parse_stan_vars(x)
+        stan_vars_dims, stan_vars_cols, stan_var_types = parse_stan_vars(x)
         self.assertEqual(len(stan_vars_dims), 0)
         self.assertEqual(len(stan_vars_cols), 0)
+        self.assertEqual(len(stan_var_types), 0)
 
     def test_parse_missing(self):
         with self.assertRaises(ValueError):
-            parse_sampler_vars(None)
+            parse_method_vars(None)
         with self.assertRaises(ValueError):
             parse_stan_vars(None)
 
-    def test_parse_sampler_vars(self):
+    def test_parse_method_vars(self):
         x = [
             'lp__',
             'accept_stat__',
@@ -606,14 +695,14 @@ class ParseVarsTest(unittest.TestCase):
             'z_init[1]',
             'z_init[2]',
         ]
-        vars_dict = parse_sampler_vars(x)
+        vars_dict = parse_method_vars(x)
         self.assertEqual(len(vars_dict), 7)
         self.assertEqual(vars_dict['lp__'], (0,))
         self.assertEqual(vars_dict['stepsize__'], (2,))
 
     def test_parse_scalars(self):
         x = ['lp__', 'foo']
-        dims_map, cols_map = parse_stan_vars(x)
+        dims_map, cols_map, _ = parse_stan_vars(x)
         self.assertEqual(len(dims_map), 1)
         self.assertEqual(dims_map['foo'], ())
         self.assertEqual(len(cols_map), 1)
@@ -622,7 +711,7 @@ class ParseVarsTest(unittest.TestCase):
         dims_map = {}
         cols_map = {}
         x = ['lp__', 'foo1', 'foo2']
-        dims_map, cols_map = parse_stan_vars(x)
+        dims_map, cols_map, _ = parse_stan_vars(x)
         self.assertEqual(len(dims_map), 2)
         self.assertEqual(dims_map['foo1'], ())
         self.assertEqual(dims_map['foo2'], ())
@@ -630,6 +719,14 @@ class ParseVarsTest(unittest.TestCase):
         self.assertEqual(cols_map['foo1'], (1,))
         self.assertEqual(cols_map['foo2'], (2,))
 
+        dims_map = {}
+        cols_map = {}
+        x = ['lp__', 'z[real]', 'z[imag]']
+        dims_map, cols_map, types_map = parse_stan_vars(x)
+        self.assertEqual(len(dims_map), 1)
+        self.assertEqual(dims_map['z'], (2,))
+        self.assertEqual(types_map['z'], BaseType.COMPLEX)
+
     def test_parse_containers(self):
         # demonstrates flaw in shortcut to get container dims
         x = [
@@ -642,7 +739,7 @@ class ParseVarsTest(unittest.TestCase):
             'phi[10]',
             'bar',
         ]
-        dims_map, cols_map = parse_stan_vars(x)
+        dims_map, cols_map, _ = parse_stan_vars(x)
         self.assertEqual(len(dims_map), 3)
         self.assertEqual(dims_map['foo'], ())
         self.assertEqual(dims_map['phi'], (10,))  # sic
@@ -672,7 +769,7 @@ class ParseVarsTest(unittest.TestCase):
         ]
         dims_map = {}
         cols_map = {}
-        dims_map, cols_map = parse_stan_vars(x)
+        dims_map, cols_map, _ = parse_stan_vars(x)
         self.assertEqual(len(dims_map), 3)
         self.assertEqual(
             dims_map['phi'],
@@ -700,7 +797,7 @@ class ParseVarsTest(unittest.TestCase):
         ]
         dims_map = {}
         cols_map = {}
-        dims_map, cols_map = parse_stan_vars(x)
+        dims_map, cols_map, _ = parse_stan_vars(x)
         self.assertEqual(len(dims_map), 2)
         self.assertEqual(
             dims_map['phi'],
@@ -715,14 +812,83 @@ class ParseVarsTest(unittest.TestCase):
 
 
 class DoCommandTest(unittest.TestCase):
-    def test_good(self):
-        retstr = do_command('ls', HERE)
-        self.assertIsNotNone(retstr)
+    def test_capture_console(self):
+        tmp = io.StringIO()
+        do_command(cmd=['ls'], cwd=HERE, fd_out=tmp)
+        self.assertTrue('test_utils.py' in tmp.getvalue())
 
     def test_exit(self):
-        args = ['bash', '/bin/junk']
-        with self.assertRaises(Exception):
-            do_command(args, HERE)
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            args = ['bash', '/bin/junk']
+            with self.assertRaises(RuntimeError):
+                do_command(args, HERE)
+
+
+class PushdTest(unittest.TestCase):
+    def test_restore_cwd(self):
+        "Ensure do_command in a different cwd restores cwd after error."
+        cwd = os.getcwd()
+        with self.assertRaises(RuntimeError):
+            with pushd(os.path.dirname(cwd)):
+                raise RuntimeError('error')
+        self.assertEqual(cwd, os.getcwd())
+
+
+class FlattenTest(unittest.TestCase):
+    def test_good(self):
+        array_3d = np.empty((200, 4, 4))
+        vals = [1.0, 2.0, 3.0, 4.0]
+        pos = [(0, 0, 0), (0, 1, 1), (0, 2, 2), (0, 3, 3)]
+        draws, chains, cols = zip(*pos)
+        array_3d[draws, chains, cols] = vals
+        flattened = flatten_chains(array_3d)
+
+        self.assertEqual(flattened.shape, (800, 4))
+        self.assertEqual(flattened[0, 0], 1.0)
+        self.assertEqual(flattened[200, 1], 2.0)
+        self.assertEqual(flattened[400, 2], 3.0)
+        self.assertEqual(flattened[600, 3], 4.0)
+
+    def test_bad(self):
+        array_2d = np.empty((200, 4))
+        with self.assertRaisesRegex(ValueError, 'Expecting 3D array'):
+            flatten_chains(array_2d)
+
+
+@pytest.mark.order(-1)
+class ShowProgressTest(unittest.TestCase):
+    # this test must run after any tests that check tqdm progress bars
+    def test_show_progress_fns(self):
+        self.assertTrue(allow_show_progress())
+        with LogCapture() as log:
+            logging.getLogger()
+            try:
+                raise ValueError("error")
+            except ValueError as e:
+                _disable_progress(e)
+        log.check_present(
+            (
+                'cmdstanpy',
+                'ERROR',
+                'Error in progress bar initialization:\n'
+                '\terror\n'
+                'Disabling progress bars for this session',
+            )
+        )
+        self.assertFalse(allow_show_progress())
+        try:
+            raise ValueError("error")
+        except ValueError as e:
+            with LogCapture() as log:
+                logging.getLogger()
+                _disable_progress(e)
+        msgs = ' '.join(log.actual())
+        # msg should only be printed once per session - check not found
+        self.assertEqual(
+            -1, msgs.find('Disabling progress bars for this session')
+        )
+        self.assertFalse(allow_show_progress())
 
 
 if __name__ == '__main__':
